#include <list>
#include "re2faTest.h"

#include "regexp/convert/ToAutomatonDerivation.h"
#include "regexp/convert/ToAutomatonGlushkov.h"
#include "regexp/convert/ToAutomatonThompson.h"
#include "automaton/convert/ToRegExpAlgebraic.h"
#include "automaton/determinize/Determinize.h"
#include "automaton/simplify/Minimize.h"
#include "automaton/simplify/Normalize.h"
#include "automaton/simplify/EpsilonRemover.h"

#include "regexp/unbounded/UnboundedRegExp.h"
#include "regexp/RegExpFromStringParser.h"

#include "automaton/FSM/NFA.h"
#include <factory/DataFactory.hpp>

CPPUNIT_TEST_SUITE_REGISTRATION( re2faTest );

void re2faTest::setUp() {
}

void re2faTest::tearDown() {
}

void re2faTest::testThompson() {
	std::string input = "a+a* b*";
	std::stringstream inputs(input);

	regexp::RegExpFromStringParser parser(inputs);
	regexp::RegExp regexp1( parser.parseValue() );

	automaton::Automaton enfa1 = regexp::convert::ToAutomatonThompson::convert(regexp1);

	regexp::RegExp regexp2( automaton::convert::ToRegExpAlgebraic::convert(enfa1) );

	automaton::Automaton enfa2 = regexp::convert::ToAutomatonThompson::convert(regexp2);

	automaton::Automaton nfa1 = automaton::simplify::EpsilonRemover::remove(enfa1);
	automaton::Automaton nfa2 = automaton::simplify::EpsilonRemover::remove(enfa2);

	automaton::Automaton dfa1 = automaton::determinize::Determinize::determinize(nfa1);
	automaton::Automaton dfa2 = automaton::determinize::Determinize::determinize(nfa2);

	automaton::Automaton mdfa1 = automaton::simplify::Minimize::minimize(dfa1);
	automaton::Automaton mdfa2 = automaton::simplify::Minimize::minimize(dfa2);

	CPPUNIT_ASSERT( mdfa1 == mdfa2);
}

void re2faTest::testGlushkov() {
	std::string input = "a+a* b*";
	std::stringstream inputs(input);

	regexp::RegExpFromStringParser parser(inputs);
	regexp::UnboundedRegExp regexp1( static_cast<const regexp::UnboundedRegExp &>( parser.parseValue().getData() ) );

	automaton::NFA nfa1 = regexp::convert::ToAutomatonGlushkov::convert(regexp1);

	regexp::UnboundedRegExp regexp2( static_cast<const regexp::UnboundedRegExp &>( automaton::convert::ToRegExpAlgebraic::convert(nfa1) ) );

	automaton::NFA nfa2 = regexp::convert::ToAutomatonGlushkov::convert(regexp2);

	automaton::DFA dfa1 = automaton::determinize::Determinize::determinize(nfa1);
	automaton::DFA dfa2 = automaton::determinize::Determinize::determinize(nfa2);

	automaton::DFA mdfa1 = automaton::simplify::Minimize::minimize(dfa1);
	automaton::DFA mdfa2 = automaton::simplify::Minimize::minimize(dfa2);

	CPPUNIT_ASSERT( mdfa1 == mdfa2);
}

void re2faTest::testBrzozowski() {
	std::string input = "a+a* b*";
	std::stringstream inputs(input);

	regexp::RegExpFromStringParser parser(inputs);
	regexp::RegExp regexp1( parser.parseValue() );

	automaton::Automaton nfa1 = regexp::convert::ToAutomatonDerivation::convert(regexp1);

	regexp::RegExp regexp2( automaton::convert::ToRegExpAlgebraic::convert(static_cast<const automaton::NFA&>(nfa1.getData())) );

	automaton::Automaton nfa2 = regexp::convert::ToAutomatonDerivation::convert(regexp2);

	automaton::DFA mdfa1_1 = automaton::determinize::Determinize::determinize(static_cast<const automaton::NFA&>(nfa1.getData()));
	automaton::DFA mdfa1_2 = automaton::simplify::Minimize::minimize(mdfa1_1);
	automaton::DFA mdfa1_3 = automaton::simplify::Normalize::normalize(mdfa1_2);

	automaton::DFA mdfa2_1 = automaton::determinize::Determinize::determinize(static_cast<const automaton::NFA&>(nfa2.getData()));
	automaton::DFA mdfa2_2 = automaton::simplify::Minimize::minimize(mdfa2_1);
	automaton::DFA mdfa2_3 = automaton::simplify::Normalize::normalize(mdfa2_2);

	CPPUNIT_ASSERT( mdfa1_3 == mdfa2_3);
}
